import WebSocket, { RawData } from "ws";
import { RPMessage } from "../utils/type";
import Manager from "../Manager";

export default abstract class ProtocolBase {

    constructor(protected id: string,
        protected ws: WebSocket,
        protected manager: Manager) {
    }

    abstract handleMessage(message: RPMessage): void;

    protected setupMsgListener() {
        this.ws.on('message', (message: RawData) => {
            try {
                const json = JSON.parse(message.toString());
                this.handleMessage(json);
            } catch (error) {
                console.error('Error parsing message:', error);
                this.sendError('0002', 'Invalid message format');
            }
        });
    }

    protected send(data: any) {
        this.ws.send(JSON.stringify(data));
    }

    protected sendError(code: string, message: string) {
        this.send({ code, data: {"_": message} });
    }

    protected sendUnknownCommand() {
        this.sendError('0003', 'Unknown command');
    }

    public onDisconnect() {
        // 处理断开连接的逻辑
        if (this.id === null) return;
        this.manager.removeUser(this.id);
        console.log(`Client disconnected ${this.id}`);
    }
}